/*
	This file is part of ldraw.

	ldraw is free software: you can redistribute it and/or modify
	it under the terms of the GNU Lesser General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	ldraw is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public License
	along with ldraw.  If not, see <http://www.gnu.org/licenses/>.
    
    Copyright (C) 2008 Lucas Hermann Negri
*/

#include "interface.h"

static void priv_register(lua_State* L, const char* name, const char* mt, 
	const luaL_reg* methods, lua_CFunction gc, lua_CFunction tostring)
{
	/* Register the methods */
	luaL_register(L, name, methods);
	
	/* Register the metatable */
	luaL_newmetatable(L, mt);
	
	/* __index */
	lua_pushliteral(L, "__index");
	lua_pushvalue(L, -3);
	lua_rawset(L, -3);
	
	/* __gc */
	lua_pushliteral(L, "__gc");
	lua_pushcfunction(L, gc);
	lua_rawset(L, -3);
	
	/* __tostring */
	lua_pushliteral(L, "__tostring");
	lua_pushcfunction(L, tostring);
	lua_rawset(L, -3);
}

static const struct luaL_reg context [] =
{
	#ifdef HAVE_GTK
	{"createFromDrawable", ldraw_context_create_from_drawable},
	{"getFromPrintContext", ldraw_context_get_from_print_context},
	{"setSourceColor", ldraw_context_set_source_color},
	{"setSourcePixbuf", ldraw_context_set_source_pixbuf},
	{"pangoLayoutPath", ldraw_pango_context_layout_path},
	{"pangoShowLayout", ldraw_pango_context_show_layout},
	{"pangoUpdateLayout", ldraw_pango_context_show_layout},
	{"pangoLayoutPath", ldraw_pango_context_layout_path},
	#endif
	{"destroy", ldraw_context_destroy},
	{"setSourceRgb", ldraw_context_set_source_rgb},
	{"setSourceRgba", ldraw_context_set_source_rgba},
	{"setSourceSurface", ldraw_context_set_source_surface},
	{"clip", ldraw_context_clip},
	{"status", ldraw_context_status},
	{"save", ldraw_context_save},
	{"restore", ldraw_context_restore},
	{"paint", ldraw_context_paint},
	{"paintWithAlpha", ldraw_context_paint_with_alpha},
	{"selectFontFace", ldraw_context_select_font_face},
	{"setFontSize", ldraw_context_set_font_size},
	{"moveTo", ldraw_context_move_to},
	{"lineTo", ldraw_context_line_to},
	{"curveTo", ldraw_context_curve_to},
	{"relMoveTo", ldraw_context_rel_move_to},
	{"relLineTo", ldraw_context_rel_line_to},
	{"relCurveTo", ldraw_context_rel_curve_to},
	{"closePath", ldraw_context_close_path},
	{"arc", ldraw_context_arc},
	{"arcNegative", ldraw_context_arc_negative},
	{"showText", ldraw_context_show_text},
	{"setLineWidth", ldraw_context_set_line_width},
	{"setLineCap", ldraw_context_set_line_cap},
	{"setLineJoin", ldraw_context_set_line_join},
	{"setDash", ldraw_context_set_dash},
	{"rectangle", ldraw_context_rectangle},
	{"stroke", ldraw_context_stroke},
	{"fill", ldraw_context_fill},
	{"setAntialias", ldraw_context_set_antialias},
	{"getAntialias", ldraw_context_get_antialias},
	{"scale", ldraw_context_scale},
	{"rotate", ldraw_context_rotate},
	{"translate", ldraw_context_translate},
	{"create", ldraw_context_create},
	{"source", ldraw_context_set_source},
	{"mask", ldraw_context_mask},
	{"maskSurface", ldraw_context_mask_surface},
	{"strokePreserve", ldraw_context_stroke_preserve},
	{"fillPreserve", ldraw_context_fill_preserve},
	{"transform", ldraw_context_transform},
	{NULL, NULL}
};

static const struct luaL_reg surface [] =
{
	{"destroy", ldraw_surface_destroy},
	{"writeToPng", ldraw_surface_write_to_png},
	{"createFromPng", ldraw_image_surface_create_from_png},
	{"create", ldraw_image_surface_create},
	{"createPdf", ldraw_surface_create_pdf},
	{"createSvg", ldraw_surface_create_svg},
	{"createPs", ldraw_surface_create_ps},
	{"getSize", ldraw_surface_get_size},
	{"status", ldraw_surface_status},
	{NULL, NULL}
};

static const struct luaL_reg pattern [] =
{
	{"createRadial", ldraw_pattern_create_radial},
	{"createLinear", ldraw_pattern_create_linear},
	{"destroy", ldraw_pattern_destroy},
	{"addColorStopRgb", ldraw_pattern_add_color_stop_rgb},
	{"addColorStopRgba", ldraw_pattern_add_color_stop_rgba},
	{NULL, NULL}
};

static const struct luaL_reg matrix [] =
{
	{"create", ldraw_matrix_create},
	{"destroy", ldraw_matrix_destroy},
	{"init", ldraw_matrix_init},
	{"initIdentity", ldraw_matrix_init_identity},
	{"invert", ldraw_matrix_invert},
	{"multiply", ldraw_matrix_multiply},
	{"rotate", ldraw_matrix_rotate},
	{"scale", ldraw_matrix_scale},
	{"transformDistance", ldraw_matrix_transform_distance},
	{"transformPoint", ldraw_matrix_transform_point},
	{"translate", ldraw_matrix_translate},
	{NULL, NULL}
};

static const struct luaL_reg ldraw [] =
{
	{"version", ldraw_version},
	{NULL, NULL}
};

int luaopen_ldraw(lua_State *L)
{
	/* Register the basic functions */
	luaL_register(L, LIB_NAME, ldraw);
	
	/* enum cairo_font_weight_t */
	lua_pushliteral(L, "FONT_WEIGHT_NORMAL"); lua_pushnumber(L, CAIRO_FONT_WEIGHT_NORMAL); lua_rawset(L, -3);
	lua_pushliteral(L, "FONT_WEIGHT_BOLD"); lua_pushnumber(L, CAIRO_FONT_WEIGHT_BOLD); lua_rawset(L, -3);
	
	/* enum cairo_font_slant_t */
	lua_pushliteral(L, "FONT_SLANT_NORMAL"); lua_pushnumber(L, CAIRO_FONT_SLANT_NORMAL); lua_rawset(L, -3);
	lua_pushliteral(L, "FONT_SLANT_ITALIC"); lua_pushnumber(L, CAIRO_FONT_SLANT_ITALIC); lua_rawset(L, -3);
	lua_pushliteral(L, "FONT_SLANT_OBLIQUE"); lua_pushnumber(L, CAIRO_FONT_SLANT_OBLIQUE); lua_rawset(L, -3);
	
	/* enum cairo_antialias_t */
	lua_pushliteral(L, "ANTIALIAS_DEFAULT"); lua_pushnumber(L, CAIRO_ANTIALIAS_DEFAULT); lua_rawset(L, -3);
	lua_pushliteral(L, "ANTIALIAS_NONE"); lua_pushnumber(L, CAIRO_ANTIALIAS_NONE); lua_rawset(L, -3);
	lua_pushliteral(L, "ANTIALIAS_GRAY"); lua_pushnumber(L, CAIRO_ANTIALIAS_GRAY); lua_rawset(L, -3);
	lua_pushliteral(L, "ANTIALIAS_SUBPIXEL"); lua_pushnumber(L, CAIRO_ANTIALIAS_SUBPIXEL); lua_rawset(L, -3);
	
	/* enum cairo_format_t */
	lua_pushliteral(L, "FORMAT_ARGB32"); lua_pushnumber(L, CAIRO_FORMAT_ARGB32); lua_rawset(L, -3);
	lua_pushliteral(L, "FORMAT_RGB24"); lua_pushnumber(L, CAIRO_FORMAT_RGB24); lua_rawset(L, -3);
	lua_pushliteral(L, "FORMAT_A8"); lua_pushnumber(L, CAIRO_FORMAT_A8); lua_rawset(L, -3);
	lua_pushliteral(L, "FORMAT_A1"); lua_pushnumber(L, CAIRO_FORMAT_A1); lua_rawset(L, -3);
	
	/* enum cairo_line_cap_t */
	lua_pushliteral(L, "LINE_CAP_BUTT"); lua_pushnumber(L, CAIRO_LINE_CAP_BUTT); lua_rawset(L, -3);
	lua_pushliteral(L, "LINE_CAP_ROUND"); lua_pushnumber(L, CAIRO_LINE_CAP_ROUND); lua_rawset(L, -3);
	lua_pushliteral(L, "LINE_CAP_SQUARE"); lua_pushnumber(L, CAIRO_LINE_CAP_SQUARE); lua_rawset(L, -3);
	
	/* enum cairo_line_join */
	lua_pushliteral(L, "LINE_JOIN_MITER"); lua_pushnumber(L, CAIRO_LINE_JOIN_MITER); lua_rawset(L, -3);
	lua_pushliteral(L, "LINE_JOIN_ROUND"); lua_pushnumber(L, CAIRO_LINE_JOIN_ROUND); lua_rawset(L, -3);
	lua_pushliteral(L, "LINE_JOIN_BEVEL"); lua_pushnumber(L, CAIRO_LINE_JOIN_BEVEL); lua_rawset(L, -3);
	
	/* enum cairo_status_t */
	lua_pushliteral(L, "STATUS_SUCCESS"); lua_pushnumber(L, CAIRO_STATUS_SUCCESS); lua_rawset(L, -3);
	lua_pushliteral(L, "STATUS_NO_MEMORY"); lua_pushnumber(L, CAIRO_STATUS_NO_MEMORY); lua_rawset(L, -3);
	lua_pushliteral(L, "STATUS_INVALID_RESTORE"); lua_pushnumber(L, CAIRO_STATUS_INVALID_RESTORE); lua_rawset(L, -3);
	lua_pushliteral(L, "STATUS_INVALID_POP_GROUP"); lua_pushnumber(L, CAIRO_STATUS_INVALID_POP_GROUP); lua_rawset(L, -3);
	lua_pushliteral(L, "STATUS_NO_CURRENT_POINT"); lua_pushnumber(L, CAIRO_STATUS_NO_CURRENT_POINT); lua_rawset(L, -3);
	lua_pushliteral(L, "STATUS_INVALID_MATRIX"); lua_pushnumber(L, CAIRO_STATUS_INVALID_MATRIX); lua_rawset(L, -3);
	lua_pushliteral(L, "STATUS_INVALID_STATUS"); lua_pushnumber(L, CAIRO_STATUS_INVALID_STATUS); lua_rawset(L, -3);
	lua_pushliteral(L, "STATUS_NULL_POINTER"); lua_pushnumber(L, CAIRO_STATUS_NULL_POINTER); lua_rawset(L, -3);
	lua_pushliteral(L, "STATUS_INVALID_STRING"); lua_pushnumber(L, CAIRO_STATUS_INVALID_STRING); lua_rawset(L, -3);
	lua_pushliteral(L, "STATUS_INVALID_PATH_DATA"); lua_pushnumber(L, CAIRO_STATUS_INVALID_PATH_DATA); lua_rawset(L, -3);
	lua_pushliteral(L, "STATUS_READ_ERROR"); lua_pushnumber(L, CAIRO_STATUS_READ_ERROR); lua_rawset(L, -3);
	lua_pushliteral(L, "STATUS_WRITE_ERROR"); lua_pushnumber(L, CAIRO_STATUS_WRITE_ERROR); lua_rawset(L, -3);
	lua_pushliteral(L, "STATUS_SURFACE_FINISHED"); lua_pushnumber(L, CAIRO_STATUS_SURFACE_FINISHED); lua_rawset(L, -3);
	lua_pushliteral(L, "STATUS_SURFACE_TYPE_MISMATCH"); lua_pushnumber(L, CAIRO_STATUS_SURFACE_TYPE_MISMATCH); lua_rawset(L, -3);
	lua_pushliteral(L, "STATUS_PATTERN_TYPE_MISMATCH"); lua_pushnumber(L, CAIRO_STATUS_PATTERN_TYPE_MISMATCH); lua_rawset(L, -3);
	lua_pushliteral(L, "STATUS_INVALID_CONTENT"); lua_pushnumber(L, CAIRO_STATUS_INVALID_CONTENT); lua_rawset(L, -3);
	lua_pushliteral(L, "STATUS_INVALID_FORMAT"); lua_pushnumber(L, CAIRO_STATUS_INVALID_FORMAT); lua_rawset(L, -3);
	lua_pushliteral(L, "STATUS_INVALID_VISUAL"); lua_pushnumber(L, CAIRO_STATUS_INVALID_VISUAL); lua_rawset(L, -3);
	lua_pushliteral(L, "STATUS_FILE_NOT_FOUND"); lua_pushnumber(L, CAIRO_STATUS_FILE_NOT_FOUND); lua_rawset(L, -3);
	lua_pushliteral(L, "STATUS_INVALID_DASH"); lua_pushnumber(L, CAIRO_STATUS_INVALID_DASH); lua_rawset(L, -3);
	lua_pushliteral(L, "STATUS_INVALID_DSC_COMMENT"); lua_pushnumber(L, CAIRO_STATUS_INVALID_DSC_COMMENT); lua_rawset(L, -3);
	lua_pushliteral(L, "STATUS_INVALID_INDEX"); lua_pushnumber(L, CAIRO_STATUS_INVALID_INDEX); lua_rawset(L, -3);
	lua_pushliteral(L, "STATUS_CLIP_NOT_REPRESENTABLE"); lua_pushnumber(L, CAIRO_STATUS_CLIP_NOT_REPRESENTABLE); lua_rawset(L, -3);
	lua_pushliteral(L, "STATUS_TEMP_FILE_ERROR"); lua_pushnumber(L, CAIRO_STATUS_TEMP_FILE_ERROR); lua_rawset(L, -3);
	lua_pushliteral(L, "STATUS_INVALID_STRIDE"); lua_pushnumber(L, CAIRO_STATUS_INVALID_STRIDE); lua_rawset(L, -3);
	
#if CAIRO_VERSION_MINOR >= 17
	lua_pushliteral(L, "STATUS_FONT_TYPE_MISMATCH"); lua_pushnumber(L, CAIRO_STATUS_FONT_TYPE_MISMATCH); lua_rawset(L, -3);
	lua_pushliteral(L, "STATUS_USER_FONT_IMMUTABLE"); lua_pushnumber(L, CAIRO_STATUS_USER_FONT_IMMUTABLE); lua_rawset(L, -3);
	lua_pushliteral(L, "STATUS_USER_FONT_ERROR"); lua_pushnumber(L, CAIRO_STATUS_USER_FONT_ERROR); lua_rawset(L, -3);
#endif
	
	/* Register the classes */
	priv_register(L, LIB_NAME".Context", "__MtCoNtExT", context,
		ldraw_context_destroy, ldraw_context_tostring); 
	
	priv_register(L, LIB_NAME".Surface", "__MtSuRfAcE", surface,
		ldraw_surface_destroy, ldraw_surface_tostring);
	
	priv_register(L, LIB_NAME".Matrix", "__MtMaTrIx", matrix,
		ldraw_matrix_destroy, ldraw_matrix_tostring);
	
	priv_register(L, LIB_NAME".Pattern", "__MtPaTtErN", pattern,
		ldraw_pattern_destroy, ldraw_pattern_tostring);
		
	/* Let ldraw table be on the top */
	lua_getfield(L, LUA_GLOBALSINDEX, LIB_NAME);
	
	return 1;
}
